@@ -9,6 +9,22 @@
 
 #define LIST_TYPES	5
 
+#ifdef HAVE_GNOME
+/* Keep the following in sync with setup.c */
+
+/* Number of columns that each mode requires */
+#define GMC_COLUMNS_BRIEF 2     /* brief view    */
+#define GMC_COLUMNS_DETAILED 4  /* detailed view */
+#define GMC_COLUMNS_CUSTOM 15   /* custom view   */
+#define GMC_COLUMNS (GMC_COLUMNS_BRIEF + GMC_COLUMNS_DETAILED + GMC_COLUMNS_CUSTOM)
+
+/* Default column widths */
+extern int default_column_width[GMC_COLUMNS];
+
+/* custom listing format */
+extern char *default_user_format;
+#endif
+
 enum list_types {
     list_full,			/* Name, size, perm/date */
     list_brief,			/* Name */
@@ -76,7 +92,7 @@ typedef struct format_e {
 typedef struct {
     Widget   widget;
     dir_list dir;		/* Directory contents */
-    
+
     int      list_type;		/* listing type (was view_type) */
     int      active;		/* If panel is currently selected */
     char     cwd [MC_MAXPATHLEN];/* Current Working Directory */
@@ -108,7 +124,7 @@ typedef struct {
     format_e *status_format;    /* Mini status format */
 
     int      format_modified;	/* If the format was changed this is set */
-    
+
     char     *panel_name;	/* The panel name */
     struct   stat dir_stat;	/* Stat of current dir: used by execute () */
 
@@ -118,15 +134,15 @@ typedef struct {
     int	     total_width;
     int	     ascent;
     int	     descent;
-    
+
     int      searching;
     char     search_buffer [256];
-   
+
     void     *port_ui;		/* UI stuff specific to each GUI port */
 
 #ifdef HAVE_GNOME
     /* These are standard GtkWidgets */
-	
+
     void *xwindow;		/* The toplevel window */
 
     void *table;
@@ -144,6 +160,9 @@ typedef struct {
     void *current_dir;		/* A WInput* */
     int estimated_total;
 
+    /* default column layout */
+    int column_width[GMC_COLUMNS];
+
     /* navigation buttons */
     void *back_b;
     void *fwd_b;
@@ -172,8 +191,14 @@ typedef struct {
     int drag_tree_timeout_id;
     file_entry *drag_tree_fe;
 
+    /* Whether the user is dragging something over the tree */
+    int drag_tree_dragging_over;
+
     void *panel_listbox;	/* container for the list */
     int is_a_desktop_panel;
+
+    /* CORBA servant */
+    void *servant;
 #endif
 } WPanel;
 
